#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

// extern bool JoystickAvailable;

static word PrintS(word yx, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        yx = PrintC(yx, c);
        ++p;
    }
    return yx;
}

void PrintStatus() 
{
    PrintS(YStep * 1 + 32 * XStep, "SCORE");
    PrintS(YStep * 5 + 32 * XStep, "HI-SCORE");
    PrintS(YStep * 9 + 32 * XStep, "STAGE");
    PrintByteNumber2(YStep * 9 + 38 * XStep, CurrentStage + 1);
    PrintS(YStep * 12 + 32 * XStep, "TIME");

    {
        static const byte[] RemainChars = {
            Char_MyFort + 2, 
            Char_MyFort + 3,
            Char_MyFort + 6,
            Char_MyFort + 7,
        };
        word yx;
        yx = YStep * 21 + 33 * XStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                ptr<byte> pChars;
                pChars = RemainChars;
                repeat (2) {
                    repeat (2) {
                        yx = Put(yx, *pChars);
                        ++pChars;
                    }
                    yx += YStep - 2 * XStep;
                }
                yx += 3 * XStep - YStep * 2;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    word yx;
    
    yx = PrintNumber5(YStep * 2 + 34 * XStep, Score);
    PrintC(yx, '0');

    yx = PrintNumber5(YStep * 6 + 34 * XStep, HiScore);
    PrintC(yx, '0');
}

void PrintTime()
{
    PrintByteNumber3(YStep * 12 + 37 * XStep, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    word yx;

    yx = YStep * 10 + 10;
    repeat (height) {
        repeat (width) {
            yx = PrintC(yx, ' ');
        }
        yx += YStep - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(YStep * 11 + 12 * XStep, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(YStep * 11 + 13 * XStep, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	B
        0x4c, 0x47, 0x45, 0x4b, 
        0x4c, 0x4b, 0x4a, 0x47, 
        0x4c, 0x43, 0x40, 0x4f, 
        0x44, 0x45, 0x45, 0x41, 
        //	A
        0x40, 0x4e, 0x4d, 0x42, 
        0x4c, 0x43, 0x40, 0x4f, 
        0x4c, 0x47, 0x45, 0x4f, 
        0x44, 0x41, 0x40, 0x45, 
        //	T
        0x44, 0x4d, 0x47, 0x41, 
        0x40, 0x4c, 0x43, 0x40, 
        0x40, 0x4c, 0x43, 0x40, 
        0x40, 0x44, 0x41, 0x40, 
        //	T
        0x44, 0x4d, 0x47, 0x41, 
        0x40, 0x4c, 0x43, 0x40, 
        0x40, 0x4c, 0x43, 0x40, 
        0x40, 0x44, 0x41, 0x40, 
        //	L
        0x4c, 0x43, 0x40, 0x40, 
        0x4c, 0x43, 0x40, 0x40, 
        0x4c, 0x43, 0x40, 0x40, 
        0x44, 0x45, 0x45, 0x41, 
        //	O
        0x48, 0x47, 0x45, 0x4b, 
        0x4c, 0x43, 0x40, 0x4f, 
        0x4c, 0x43, 0x40, 0x4f, 
        0x40, 0x45, 0x45, 0x41, 
        //	T
        0x44, 0x4d, 0x47, 0x41, 
        0x40, 0x4c, 0x43, 0x40, 
        0x40, 0x4c, 0x43, 0x40, 
        0x40, 0x44, 0x41, 0x40, 
    };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        word yx;
        yx = YStep * 8 + 2 * XStep;
        p = TitleBytes;
        repeat (7) {
            repeat (4) {
                repeat (4) {
                    yx = Put(yx, *p);
                    ++p;
                }
                yx += YStep - 4 * XStep;
            }
            yx += 4 * XStep - YStep * 4;
        }
    }
    PrintS(YStep * 20 + 9 * XStep, "PUSH SPACE KEY");
    PrintS(YStep * 21 + 9 * XStep, "OR SHIFT KEY");
    // {
    //     byte c;
    //     word v;
    //     v = yx+YStep*8;
    //     c = 0;
    //     repeat (16) {
    //         repeat (16) {
    //             v = Put(v, c);
    //             ++c;
    //         }
    //         v += YStep - 16;
    //     }
    // }
}


void DrawFence()
{
    word top, bottom;
    top = 0;
    bottom = YStep * 23;
    repeat(StageWidth) {
        top = Put(top, Char_Fence);
        bottom = Put(bottom, Char_Fence + 1);
    }
}